#!/usr/bin/python3
"""This module implements some helper functions and a simple SCM tool."""

import devpipeline_core.plugin
import devpipeline_core.toolsupport

import devpipeline_scm


def _nothing_scm(current_target):
    # Unused variables
    del current_target

    class _NothingScm:
        def checkout(self, repo_dir):
            # pylint: disable=missing-docstring
            pass

        def update(self, repo_dir):
            # pylint: disable=missing-docstring
            pass

    return _NothingScm()


_NOTHING_SCM = (_nothing_scm, "Do nothing.")


def _no_scm_check(configuration, error_fn):
    for component_name in configuration.components():
        component = configuration.get(component_name)
        if "scm" not in component:
            error_fn("No scm declared in {}".format(component_name))


def _make_scm(current_target):
    """
    Create an Scm for a component.

    Arguments
    component - The component being operated on.
    """
    return devpipeline_core.toolsupport.tool_builder(
        current_target["current_config"], "scm",
        devpipeline_scm.SCMS, current_target)


def scm_task(current_target):
    """
    Update or a local checkout.

    Arguments
    target - The target to operate on.
    """
    try:
        scm = _make_scm(current_target)
        src_dir = current_target["current_config"].get("dp.src_dir")
        scm.checkout(src_dir)
        scm.update(src_dir)
    except devpipeline_core.toolsupport.MissingToolKey as mtk:
        current_target["executor"].warning(mtk)



#!/usr/bin/python3

"""
Main module for devpipeline_scm.  It provides SCMS, a dictionary with all
detected scm plugins.
"""

import devpipeline_core.plugin

SCMS = devpipeline_core.plugin.query_plugins('devpipeline.scms')


class _SimpleScm(devpipeline_core.toolsupport.SimpleTool):

    """This class is a simple SCM tool."""

    def __init__(self, real, current_target):
        super().__init__(current_target, real)

    def checkout(self, repo_dir):
        """This function checks out source code."""
        self._call_helper("Checking out", self.real.checkout,
                          repo_dir)

    def update(self, repo_dir):
        """This funcion updates a checkout of source code."""
        self._call_helper("Updating", self.real.update,
                          repo_dir)


def make_simple_scm(real_scm, configuration):
    """
    Create an Scm instance that leverages executors.

    Arguments:
    real_scm - a class instance that provides an Scm interface
    configuration - the configuration for the Scm target
    """
    return _SimpleScm(real_scm, configuration)



#!/usr/bin/python3
"""This modules does the checkout of code from SCM."""

import argparse

import devpipeline_configure.cache
import devpipeline_core.command

import devpipeline_scm
import devpipeline_scm.scm


def _list_scms():
    for scm in sorted(devpipeline_scm.SCMS):
        print("{} - {}".format(scm, devpipeline_scm.SCMS[scm][1]))


_MAJOR = 0
_MINOR = 3
_PATCH = 0

_STRING = "{}.{}.{}".format(_MAJOR, _MINOR, _PATCH)


class CheckoutCommand(devpipeline_core.command.TargetCommand):
    """
    Provide the checkout command to dev-pipeline.
    """

    def __init__(self, config_fn):
        super().__init__(config_fn=config_fn,
                         prog="dev-pipeline checkout",
                         description="Checkout repositories")
        self.add_argument("--list-scms", action='store_true',
                          default=argparse.SUPPRESS,
                          help="List the available scm tools")
        self.enable_dependency_resolution()
        self.enable_executors()
        self.set_tasks([devpipeline_scm.scm.scm_task])
        self.set_version(_STRING)
        self.helper_fn = lambda: super(CheckoutCommand, self).process()

    def setup(self, arguments):
        if "list_scms" in arguments:
            self.helper_fn = _list_scms

    def process(self):
        self.helper_fn()


def main(args=None, config_fn=devpipeline_configure.cache.update_cache):
    # pylint: disable=missing-docstring
    checkout = CheckoutCommand(config_fn)
    devpipeline_core.command.execute_command(checkout, args)


_SCM_COMMAND = (main, "Checkout the proper version of the source tree.")

if __name__ == '__main__':
    main()
